{	**************************************************************

	This table converts scales from -7 to +7 in both x and y to an
	angle. Design both x and y offsets, and scale down until they
	are both < 7. Then build the byte as follow

	Bit 7 	: Y sign
	Bit 4-6 : Y Value
	Bit 3 	: X sign
	Bit 2-0 : X Value

	The resulting value is an angle from 0..255 ; a scaled angle
	calculated as Angle / 360 * 256.

	************************************************************** }

program AngleTab;

uses Graph;

var ATable:Array[0..255] of Byte;

function ArcTangent(x,y:Real):Real;
var a:Real;
begin
	if y = 0 then y := 0.00001;
	a := (ArcTan(x/y) * 360 / 2 / Pi+0.5);
	if y < 0 then a := 180 + a;
	if a < 0 then a := 360 + a;
	ArcTangent := a;
end;

function ConvI(x:Integer):String;
var s:String;
begin
	Str(x,s);
	ConvI := s;
end;

procedure DrawLine(xc,yc:Integer);
var Deg,Rad:Real;
	n,x,y:Integer;
	s:String;
	Deg256:Real;
begin
	Deg := ArcTangent(yc,xc);
	Rad := Deg / 360.0 * 2 * Pi;
	Deg256 := Deg / 360.0 * 256.0;
	x := Trunc(Cos(Rad) * 260) + 320;
	y := -Trunc(Sin(Rad) * 150) + 175;
	SetColor(Green);
	Line(320,175,x,y);
	SetColor(Yellow);
	s := ConvI(Trunc(Deg256))+','+ConvI(xc)+','+ConvI(yc);
	OutTextXY(x-TextWidth(s) div 2,y,s);
	n := 0;
	n := Abs(xc) + Abs(yc) * 16;
	if xc < 0 then n := n + 8;
	if yc < 0 then n := n + 128;
    ATable[n] := Round(Deg256);
end;


var GraphDriver, GraphMode:Integer;
	i,xc,yc,r:Integer;
	f:Text;
	fb:File of Byte;
begin
	GraphDriver := EGA;GraphMode := EGAHi;
	InitGraph(GraphDriver,GraphMode,'.');
	SetGraphMode(GraphMode);
	for i := 0 to 255 do ATable[i] := $FF;
	r := 7;
	for xc := -r to r do
		for yc := -r to r do
			DrawLine(xc,yc);
	Assign(f,'_ARCTAN.INC');
	Rewrite(f);
	WriteLn(f,';');
	WriteLn(f,';     Table for converting arctangent values to 256 unit degrees');
	WriteLn(f,';');
	WriteLn(f,'OffsetToAngle:');

	for i := 0 to 255 do
		begin
			if ((i and $F0) = $80) then
				ATable[i] := ATable[i and $7F];
			if ((i and $0F) = $08) then
				ATable[i] := ATable[i and $F7];
		end;

	for i := 0 to 255 do
		begin
			Write(f,'    .db   ',ATable[i]:3,' ; [',i:3,']');
				begin
					Write(f,'  (');
					if (i and 8) <> 0 then Write(f,'-') else Write(f,'+');
					Write(f,i and 7,',');
					if (i and 128) <> 0 then Write(f,'-') else Write(f,'+');
					Write(f,(i div 16) and 7,')');
					Write(f,'  ',Round((ATable[i] * 360.0)/256):4,' degrees.');
				end;
			WriteLn(f);
		end;
	Close(f);
	Assign(fb,'ARCTAN.BIN');
	Rewrite(fb);
	for i := 0 to 255 do Write(fb,ATable[i]);
	Close(fb);
	if ParamCount <> 1 then ReadLn;
	CloseGraph;
end.
